/*
HTML.h
*/

#ifndef __HTML_H
#define __HTML_H

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include "Config.h"
#include "Utils.h"
#include "XSocket.h"
#include "XVector.h"
#include "XString.h"
#include "Language.h"
#include "UserOptions.h"
#include "Addressbook.h"

class HTML
  {
  private:
    FILE *fout;
    int iout;
    long MAXMSGSFORPAGE;
    XSocket *OneSocket;
    XString XBuffer;
    void PrintStatusLine (const char *sl, const char *user);
  public:
    HTML (FILE *FR);
    HTML (int FR);
    HTML (XSocket *aXS);
    virtual ~HTML (void);
    void setMAXMSGSFORPAGE (long maxmsgs);
    void WRITE (char *fmt, ...);    
    void pcWRITE (char *pc);
    void pcWRITEBuffer (char *pc, xulong len);
    void Head (const char *contenttype, const char *name, const char *filename);
    void HeadInline (const char *contenttype, const char *filename, xulong len);
    void HTMLBegin (void);
    void Title (char *title, char *txtcolor, char *bgcolor);
    void HTMLEnd (void);
    void EmulatePageNotFound (void);    
    void BeginTable (int border, int cellpadding, int cellspacing, int widthpercent);
    void EndTable (void);
    void BeginTR (void);
    void BeginTR (char *bgcolor);
    void EndTR (void);
    void BeginTD (void);
    void BeginTD (char *bgcolor);
    void BeginTD (char *bgcolor, char *align);            
    void BeginTD (char *bgcolor, char *align, int colspan);    
    void BeginTD (bool nowrap, int widthpercent, int rowspan);    
    void BeginTD (int widthpercent, char *facefont, int sizefont, char *align);    
    void BeginTD (bool nowrap, int widthpercent, int rowspan, char *facefont, int sizefont, char *align);
    void EndTD (void);
    void EmptyCellx1 (void);
    void EmptyCellx2 (void);
    void BeginFont (char *face, int size, char *color);    
    void EndFont (void);
    void BeginBold (void);
    void EndBold (void);
    void BeginItalic (void);
    void EndItalic (void);
    void BeginUnderline (void);
    void EndUnderline (void);
    void BR (void);
    void BeginNOBR (void);
    void EndNOBR(void);
    void H1 (char *linea);
    void H2 (char *linea);
    void H3 (char *linea);
    void H4 (char *linea);
    void H5 (char *linea);
    void Label (char *linea);
    void RedLabel (char *linea);
    void GreenLabel (char *linea);
    void URL (char *caption, char *url);
    void Image (char *urlimage, char *label);
    void URLIMAGE (char *label, char *url, char *urlimage);
    void PrintString (char *linea);
    void pr (char *linea);    
    void Redirect (char *newurl);
    void Linea (void);
    void Error (char *error);
    void Msg (char *msg);
    void ErrorPage (char *error);
    void ErrorPageBack (char *stlang, char *user, char *connid, UserOptions *uo, char *error, int way);
    void MsgPage (char *msg, char *lang);    
    void soErrorPage (char *error);
    //void OkPage (char *okmsg);
    void OkPage (char *stlang, char *user, char *connid, UserOptions *uo, char *okmsg);
    void Ok (char *linea);
    void BeginPre (void);
    void EndPre (void);
    void BeginCenter (void);
    void EndCenter (void);
    void BeginFormPost (char *action);
    void BeginFormPostMultipartFormData (char *action);    
    void BeginFormGet (char *action);
    void EndForm (void);
    void FormHidden (char *name, char *value);
    void FormHidden (char *name, int value);    
    void FormEdit (int max, int size, char *name, char *value);
    void FormFile (int max, int size, char *name, char *value, char *accept);
    void FormTextArea (int rows, int cols, char *name, char *value);    
    void FormPassword (int max, int size, char *name, char *value);
    void FormCheckbox (char *name, char *value, bool checked);
    void FormImage (char *name, char *src);
    void FormButton (char *name, char *value);
    void FormReset (char *value);
    void FormBeginSelect (char *name);
    void FormAddOptionSelect (bool selected, const char *value, const char *label);
    void FormEndSelect ();
    void dumpSL (StringList *SL);
    void dumpPChar (char *pc);    
    void BotoneraDisplayMsg (char *lang, char *user, char *connid, UserOptions *uo, bool isdeleted);    
    void BotoneraMailboxIndex (char *lang, char *user, char *connid, UserOptions *uo);
    void BotoneraComun (int cmdactual, char *lang, char *user, char *connid, UserOptions *uo);
    void BotoneraAttachs (char *lang, char *user, char *connid, UserOptions *uo);
    void BotoneraComposeMsg (int thecmd, char *lang, char *user, char *connid, UserOptions *uo);
    void LineaLanguage (void);
    char *WorkFlags (const char *flags);
    void PrintIndexPage (char *lang, char *mailbox, char *user, char *connid, 
                         StringList *SL, StringList *SL2, UserOptions *uo);
    void PrintMessageHeader 
                      (char *lang, char *user, char *connid, 
                       long nummsgs, char *aflags, char *ato, char *afrom, 
                       char *asubject, char *adate, char *asize,
                       StringList *SLAttach, UserOptions *uo);
    void dumpAttach (char *chunk, xulong len, StringList *SLAttachInfo);
    void dumpRawAttachPC (char *pcheader, xulong lenheader, char *pcbody, xulong lenbody);
    void dumpRawAttachSL (StringList *SLAttachHeader, char *pcbody, xulong lenbody);
    void dumpAttachWithoutHead (char *chunk, xulong len);
    void PrintConfirmPage (char *stcmd, char *stlang, char *mailbox, char *user, char *connid, int validator, int pageid);
    void PrintMailboxesPage (char *lang, char *mailbox, char *user, char *connid, 
                             StringList *SLMailboxes, UserOptions *uo);    
    void PrintComposeMsgPage (int thecmd, int subcmd, char *lang, char *user, char *connid, 
                              char *to, char *cc, char *bcc, char *subject, 
                              char *replyto, char *from, char *date,
                              char *firsttext,
                              UserOptions *uo);
    void PrintShowSignature (char *stlang, char *user, char *connid, long actualpos, 
                             char *signature, UserOptions *uo);
    void PrintShowOptions (char *stlang, char *user, char *connid, UserOptions *uo);
    void PrintInvalidPage (char *stlang, char *user, char *connid, UserOptions *uo);
    void PrintShowAttachs (char *stlang, char *user, char *connid, UserOptions *uo);
    void PrintAddBook_Show (int thecmd, char *stlang, char *user, char *connid, UserOptions *uo, Addressbook *ab, int sortby);
    void PrintAddBook_EditEntry (int thecmd, char *stlang, char *user, char *connid, UserOptions *uo, Addressbook *ab, int numentry);
    };
  
#endif  
  
