/*           
UserOptions.cc
*/

#include "UserOptions.h"

UserOptions::UserOptions (const char *abasepath, const char *auser)
  {
  validator = 0;
  InitStatusLine ();
  user = XString (auser);
  basepath = XString (abasepath);
  userdir = XString (ReturnUsersDir (abasepath, auser));
  fnuseroptions = XString (ReturnUserOptionsFileName(abasepath, auser));
  fnsignature = XString (ReturnSignatureFileName(abasepath, auser));
  fnsavedmsg  = XString (ReturnUserSavedMsgFileName (abasepath, auser));
  userattdir = XString (ReturnUserAttachsDir(abasepath, auser));
  signature = XString ("");
  sort_ab = 0;
  CurrentPosition = -1;
  strcpy (MAILDOMAIN, "");
  strcpy (IMAPSERVER, "");
  strcpy (SMTPSERVER, "");
  pageid = -1;
  lastcmd = -1;
  MAILBOXSIZE = -1;
  BeginPage = 1;
  LoadOptions ();
  }
  
UserOptions::~UserOptions ()  
  {
  }  

void UserOptions::setMailDomain (const char *amaildomain)
  {
  strcpy (MAILDOMAIN, amaildomain);
  }
  
char *UserOptions::getMailDomain (void)
  {
  return xstrdup (MAILDOMAIN);
  }      
  
void UserOptions::setImapServer (const char *aimapserver)
  {
  strcpy (IMAPSERVER, aimapserver);
  }
  
char *UserOptions::getImapServer (void)
  {
  return xstrdup (IMAPSERVER);
  }        

void UserOptions::setSMTPServer (const char *asmtpserver)
  {
  strcpy (SMTPSERVER, asmtpserver);
  }
  
char *UserOptions::getSMTPServer (void)
  {
  return xstrdup (SMTPSERVER);
  }

void UserOptions::setCurrentPosition (long pos, long total)
  {
  CurrentPosition = pos;
  TotalMsgs       = total;
  if (CurrentPosition == 0) CurrentPosition = TotalMsgs;
  if (CurrentPosition > TotalMsgs) CurrentPosition = TotalMsgs;
  if (CurrentPosition < 1) CurrentPosition = 1;
  }

long UserOptions::getCurrentPosition (void)
  {
  return CurrentPosition;
  }
  
long UserOptions::getBeginPage (void)
  {
  long p;
  ldiv_t ld;
  if (getMsgsPerIndexPage() == 0)
    {
    BeginPage = 1;
    }
  else
    {
    ld = ldiv (CurrentPosition - 1, getMsgsPerIndexPage());
    p = ld.quot * getMsgsPerIndexPage();
    BeginPage = p + 1;
    }
  return BeginPage;
  }        

long UserOptions::getTotalMsgs (void)
  {
  return TotalMsgs;
  }

void UserOptions::setMailboxSize (long asize)
  {
  MAILBOXSIZE = asize;
  }
  
long UserOptions::getMailboxSize (void)    
  {
  return MAILBOXSIZE;
  }      

void UserOptions::InitStatusLine (void)
  {
  strcpy (StatusLine, HTML_SP);
  }

const char *UserOptions::getUsername (void)
  {
  return user.cstr();
  }  

const char *UserOptions::getAttachDir (void)
  {
  return userattdir.cstr();
  }  

const char *UserOptions::getStatusLine (void)
  {
  return StatusLine;
  }  

void UserOptions::setStatusLine (char *newstatusline, char *htmlcolor)
  {
  sprintf (StatusLine, "<FONT face=\"%s\" color=%s>%s</FONT>", STATUSFACEFONT, htmlcolor, newstatusline);
  }
        
int UserOptions::getValidator (void)
  {
  return validator;
  }  
  
void UserOptions::IncValidator (void)
  {
  ++validator;
  }  

void UserOptions::setInsertSignatureInNewMsg (bool value)
  {
  InsertSignatureInNewMsg = value;
  }
  
bool UserOptions::getInsertSignatureInNewMsg (void)
  {
  return InsertSignatureInNewMsg;
  }  
  
void UserOptions::setMsgsPerIndexPage (long value)
  {
  if (value < 0) MsgsPerIndexPage = DEFOPT_MSGSPERINDEXPAGE;
  else MsgsPerIndexPage = value;
  }

long UserOptions::getMsgsPerIndexPage (void)
  {
  return MsgsPerIndexPage;
  }
  
void UserOptions::setPersonalName (char *value)
  {
  personalname = XString (value);
  }

char *UserOptions::getPersonalName (void)
  {
  return (char *)(personalname.cstr());
  }  
  
void UserOptions::setSaveMsgSentMail (bool value)
  {
  SaveMsgSentMail = value;
  }
  
bool UserOptions::getSaveMsgSentMail (void)
  {
  return SaveMsgSentMail;
  }  
  
void UserOptions::setReplyFrom (bool value)
  {
  ReplyFrom = value;
  }
  
bool UserOptions::getReplyFrom (void)
  {
  return ReplyFrom;
  }    
  
void UserOptions::setForwAttachs (bool value)
  {
  ForwAttachs = value;
  }
  
bool UserOptions::getForwAttachs (void)
  {
  return ForwAttachs;
  }      
  
void UserOptions::setWidthWriteArea (int value)
  {
  WidthWriteArea = value;
  }
  
int UserOptions::getWidthWriteArea (void)
  {
  return WidthWriteArea;
  }  
  
void UserOptions::setHeightWriteArea (int value)
  {
  HeightWriteArea = value;
  }
  
int UserOptions::getHeightWriteArea (void)
  {
  return HeightWriteArea;
  }    
  
char *UserOptions::getSignature (void)
  {
  return (char *)(signature.cstr());  
  }

bool UserOptions::SaveOptions (char *tmpfilename)
  {
  TBuffer abuf;  
  FILE *fr, *fo, *fs;
  fr = fopen (tmpfilename, "r"); if (fr == NULL) return false;
  fo = fopen ((char *)(fnuseroptions.cstr()), "w"); if (fo == NULL) return false;
  fgets (abuf, CMAXBUFFER, fr); fprintf (fo, "%s", abuf);               //msg per index page
  fgets (abuf, CMAXBUFFER, fr); fprintf (fo, "%s", abuf);               //sign new messages
  fgets (abuf, CMAXBUFFER, fr); fprintf (fo, "%s", abuf);               //personal name
  fgets (abuf, CMAXBUFFER, fr); fprintf (fo, "%s", abuf);               //save msg in sent mail
  fgets (abuf, CMAXBUFFER, fr); fprintf (fo, "%s", abuf);               //reply from or reply-to
  fgets (abuf, CMAXBUFFER, fr); fprintf (fo, "%s", abuf);               //forward attachs
  fgets (abuf, CMAXBUFFER, fr); fprintf (fo, "%s", abuf);               //height write area
  fgets (abuf, CMAXBUFFER, fr); fprintf (fo, "%s", abuf);               //width write area  
  fclose (fo);
  fs = fopen ((char *)(fnsignature.cstr()), "w"); if (fs == NULL) return false;
  while (fgets (abuf, CMAXBUFFER, fr) != NULL)             //signature
    {
    fprintf (fs, "%s", abuf);  
    }
  fclose (fs);
  fclose (fr);
  DeleteFile (tmpfilename);
  LoadOptions ();
  return true;
  }  

void UserOptions::LoadOptions (void)
  {
  if (FileExists ((char *)fnuseroptions.cstr()) == false)
    {
    setMsgsPerIndexPage (DEFOPT_MSGSPERINDEXPAGE);
    setInsertSignatureInNewMsg (DEFOPT_INSERTSIGNATUREINNEWMSG);
    setPersonalName ("");
    setSaveMsgSentMail (DEFOPT_SAVEMSGSENTMAIL);
    setReplyFrom  (DEFOPT_REPLYFROM);
    setForwAttachs (DEFOPT_FORW_ATTACHS);
    setWidthWriteArea (DEFOPT_WIDTHWRITEAREA);
    setHeightWriteArea (DEFOPT_HEIGHTWRITEAREA);
    }
  else
    {
    FILE *fr;
    TBuffer tb;
    int nummmsgsperpage, widthwritearea, heightwritearea;
    fr = fopen ((char *)fnuseroptions.cstr(), "r");
    fgets (tb, CMAXBUFFER, fr); QuitaRetornoCarroDeLinea(tb); 
    nummmsgsperpage = xatoidef(tb, DEFOPT_MSGSPERINDEXPAGE); if (nummmsgsperpage < 0) nummmsgsperpage = DEFOPT_MSGSPERINDEXPAGE; setMsgsPerIndexPage(nummmsgsperpage);
    fgets (tb, CMAXBUFFER, fr); QuitaRetornoCarroDeLinea(tb); setInsertSignatureInNewMsg(xatoidef(tb, DEFOPT_INSERTSIGNATUREINNEWMSG));
    fgets (tb, CMAXBUFFER, fr); QuitaRetornoCarroDeLinea(tb); setPersonalName (tb);
    fgets (tb, CMAXBUFFER, fr); QuitaRetornoCarroDeLinea(tb); setSaveMsgSentMail(xatoidef(tb, DEFOPT_SAVEMSGSENTMAIL));
    fgets (tb, CMAXBUFFER, fr); QuitaRetornoCarroDeLinea(tb); setReplyFrom (xatoidef(tb, DEFOPT_REPLYFROM));
    fgets (tb, CMAXBUFFER, fr); QuitaRetornoCarroDeLinea(tb); setForwAttachs(xatoidef(tb, DEFOPT_FORW_ATTACHS));
    fgets (tb, CMAXBUFFER, fr); QuitaRetornoCarroDeLinea(tb); 
    heightwritearea = xatoidef(tb, DEFOPT_HEIGHTWRITEAREA); if (heightwritearea < 2) heightwritearea = DEFOPT_HEIGHTWRITEAREA; setHeightWriteArea(heightwritearea);    
    fgets (tb, CMAXBUFFER, fr); QuitaRetornoCarroDeLinea(tb); 
    widthwritearea = xatoidef(tb, DEFOPT_WIDTHWRITEAREA); if (widthwritearea < 2) widthwritearea = DEFOPT_WIDTHWRITEAREA; setWidthWriteArea(widthwritearea);
    fclose (fr);
    }
  if (FileExists ((char *)(fnsignature.cstr())) == true) 
    {
    TBuffer abuf;
    FILE *fr;
    fr = fopen ((char *)fnsignature.cstr(), "r");
    signature = "";
    while (fgets (abuf, CMAXBUFFER, fr) != NULL)
      {
      signature += abuf;
      }
    fclose (fr);
    }
  else 
    {
    signature =  XString("--\n********************************\n");  
    signature +=          user;
    signature +=           "\n********************************\n";
    }
  }  

int UserOptions::numAttachs (void)
  {
  return SLAttachList.Count ();
  }
 
char *UserOptions::getAttachInfo (int p)
  {
  if (p >= SLAttachListInfo.Count ()) return "";
  return ((char *)SLAttachListInfo.getString (p).cstr());
  }
 
char *UserOptions::getFileNameAttach (int p)
  {
  if (p >= SLAttachList.Count ()) return "";
  return ((char *)SLAttachList.getString (p).cstr());
  }      

char *UserOptions::getTrueFileNameAttach (int p)
  {
  if (p >= SLAttachList.Count ()) return "";
  XString XS;
  XS = XString (ReturnUserAttachsDir(basepath.cstr(), user.cstr())); XS += "/"; XS += getFileNameAttach(p);
  return xstrdup((char *)XS.cstr());
  }

int UserOptions::addFileNameAtach (char *newfile, char *info)
  {
  if (numAttachs () >= MAXNUMATTACHSINSEND) {return 0;}
  if (IsEmpty(newfile)) {return 0;}
  XString XS;
  XS = XString (ReturnUserAttachsDir(basepath.cstr(), user.cstr())); XS += "/"; XS += newfile;  
  if (!FileExists (XS.cstr())) {return 0;}
  if (FileSize (XS.cstr()) <= 0) {return 0;}
  if (!SLAttachList.contains(newfile)) 
    {
    SLAttachList.Add (newfile);
    //2 IMAGE/GIF 154387 bytes, "un comentario", "logo.gif"
    SLAttachListInfo.Add (info);
    return 1;
    }
  else return 0;  
  }

//SUPONGO QUE LA LISTA A BORRAR VIENE ORDENADA
// p.e. 1,5,8,40
int UserOptions::delFileNameAtach (char *attachnumbers)
  {
  if (attachnumbers == NULL) return 0;
  TBuffer abuf;
  char c, letra[2];
  int res, totaltodel;
  StringList SLtoDel;
  letra[1] = '\0';
  abuf[0]  = '\0';
  SLtoDel.Clear();
  for (int i=0; i < (xsint)strlen(attachnumbers); ++i)
    {
    c = attachnumbers[i];
    if (c != ',')
      {
      letra[0] = c;
      strcat (abuf, letra);
      }
    else
      {
      res = xatoidef (abuf, -1);
      if (res > -1) SLtoDel.Add(abuf);
      abuf[0] = '\0';
      }
    }
  if ((xsint)strlen(abuf) > 0)
    {
    res = xatoidef (abuf, -1);
    if (res > -1) SLtoDel.Add(abuf);
    }  
  totaltodel = SLtoDel.Count();  
  for (int k = SLtoDel.Count() - 1;  k >= 0; --k)
    {
    int R;
    R = atoi(SLtoDel.getString(k).cstr());
    XString XS;
    XS = XString (ReturnUserAttachsDir(basepath.cstr(), user.cstr())); 
    XS += "/"; XS += SLAttachList.getString(R).cstr();
    DeleteFile (XS.cstr());
    SLAttachList.removeElementAt (R);
    SLAttachListInfo.removeElementAt (R);
    } 
  return totaltodel;
  }
  
void UserOptions::delAllAttachs (void)
  {
  for (int i = 0; i < SLAttachList.Count(); ++i)
    {
    XString XS;
    XS = XString (ReturnUserAttachsDir(basepath.cstr(), user.cstr())); 
    XS += "/"; XS += SLAttachList.getString(i).cstr();
    DeleteFile (XS.cstr());
    }     
  SLAttachList.Clear();  
  SLAttachListInfo.Clear();
  }  

void UserOptions::setSort_ab (int value)
  {
  sort_ab = value;
  }
  
int UserOptions::getSort_ab (void)
  {
  return sort_ab;
  }                  
  
char *UserOptions::getfnSavedMsg (void)
  {
  return ((char *)fnsavedmsg.cstr());
  }
  
void UserOptions::setLastCmd (int value)
  {
  lastcmd = value;
  }
  
int UserOptions::getLastCmd (void)
  {
  return lastcmd;
  }                    

/*Vuelca por pantalla la pila*/
void UserOptions::vuelcaPila (void)
  {
  printf ("***************STACK SIZE=%d*******************\n", PilaCmds.Count());
  for (int j = PilaCmds.Count () - 1; j >= 0; --j)
    {
    printf ("---%d %s---\n", j, PilaCmds.getString(j).cstr());
    }
  }

/*Ajusto como pageid la que me llega por el cgi*/  
void UserOptions::setPageId (int value)
  {
  pageid = value;
  if ((pageid < PilaCmds.Count ()) && (pageid > -1) && (PilaCmds.Count () > 1))
    {
    for (int j = PilaCmds.Count () - 1; j >= pageid; --j)
      {
      PilaCmds.removeElementAt(j);
      }
    }
  }

/*Devuelvo una nueva pageid para la pagina nueva*/
int UserOptions::getPageId (void)
  {
  return pageid + 1;
  }    

void UserOptions::addLastStringCmd (char *stcmd)
  {
  PilaCmds.Add(stcmd); 
  //vuelcaPila();      
  }

/*Devuelve el comando back correspondiente a este pageid*/  
/*La pila va desde 0 a PilaCmds.Count () - 1*/
char *UserOptions::getLastStringCmd (int cmdactual, char *lang, char *connid, bool withhtml)
  {
  TBuffer defbuf;
  TSBuffer beghtml, endhtml, linea, aconnid, stlang, ipclient, stcmd, parm1, parm2, parm3, parm4, allparms;
  int pagprevia, cmd, otropageid, cmdcomodin, cmdcomodin2, avalidator, alastcmd;
  
  if (withhtml == true) {strcpy (beghtml, "<A HREF=\""); strcpy (endhtml, "\">");}
  else {strcpy (beghtml, ""); strcpy (endhtml, "");}  
  
  sprintf (defbuf, "%s%s/mb_index//%s/%s/%s/%d/%d/%d/%ld%s", 
           beghtml, PATH_POSTMAN, 
           lang, (char *)user.cstr(), connid, getValidator(), pageid, cmdactual, getCurrentPosition(), 
           endhtml);
  pagprevia = pageid;
  cmd = CMD_MB_INDEX;
  do
    {
    --pagprevia;
    if (pagprevia >= 0)
      {
      strcpy (linea, PilaCmds.getString(pagprevia).cstr());
      sscanf (linea, "%s %d %d %d %s %s %s %s %s %s %s", aconnid, &otropageid, &avalidator, &alastcmd, stlang, ipclient, stcmd, parm1, parm2, parm3, parm4);
      sscanf (linea, "%s %d %d %d %s %s %s %[^]]s",      aconnid, &otropageid, &avalidator, &alastcmd, stlang, ipclient, stcmd, allparms);
      cmd = TranslateCmd (stcmd);
      cmdcomodin  = CMD_AB_ADDTO; //Uno cualquiera fijo
      cmdcomodin2 = CMD_AB_ADDTO; //idem
      if (cmdactual == CMD_MG_DISPLAY) {cmdcomodin = CMD_MG_DISPLAY;}
      else if (cmdactual == CMD_MG_DELDISPLAY) {cmdcomodin = CMD_MG_DELDISPLAY;}
      else if (cmdactual == CMD_AB_ADDTO)    {cmdcomodin = CMD_AB_SHOWALL;  cmdcomodin2 = CMD_AB_FROM_CM;}
      else if (cmdactual == CMD_AB_ADDCC)    {cmdcomodin = CMD_AB_SHOWALL;  cmdcomodin2 = CMD_AB_FROM_CM;}
      else if (cmdactual == CMD_AB_ADDBCC)   {cmdcomodin = CMD_AB_SHOWALL; cmdcomodin2 = CMD_AB_FROM_CM;}
      else if (cmdactual == CMD_CM_SAVE)     {cmdcomodin = CMD_CM;}
      else if (cmdactual == CMD_CM_CANCEL)   {cmdcomodin = CMD_CM;}      
      else if (cmdactual == CMD_CM_CLEANALL) {cmdcomodin = CMD_CM; cmdcomodin2 = CMD_CM_FORWARD;}      
      }                                   
    }
  while (((cmd == CMD_AB_ADDTO)     || (cmd == CMD_AB_ADDCC)    || (cmd == CMD_AB_ADDBCC)      || 
          (cmd == CMD_AB_SAVEENTRY) || (cmd == CMD_AB_NEWENTRY) || (cmd == CMD_AB_TAKEADDRESS) || 
          (cmd == CMD_AB_EDITENTRY) ||
          (cmd == CMD_CM_CANCEL)    || (cmd == CMD_CM_CLEANALL) || (cmd == CMD_CM_SAVE)        || 
          (cmd == CMD_CONF_ACT)     ||
          (cmd == CMD_MB_EXPUNGE)   || 
          (cmd == CMD_AT_ERASE)     || (cmd == CMD_AT_SHOW)     || (cmd == CMD_AT_SAVE)        ||
          (cmd == CMD_MG_DELDISPLAY)||
          (cmd == cmdcomodin)       || (cmd == cmdcomodin2)) 
      && (pagprevia >= 0));      
  if (pagprevia < 0) pagprevia = 0;       
  switch (cmd)
    {
    case CMD_AB_FROM_CM:
      {
      sprintf (defbuf, "%s%s/ab/ab_from_cm/%s/%s/%s/%d/%d/%d/%ld%s", beghtml, PATH_POSTMAN, stlang,
               (char *)user.cstr(), connid, avalidator, otropageid, cmdactual, xatoldef (parm1, 1), endhtml);
      break;
      }           
    case CMD_AB_SHOWALL:
      {
      sprintf (defbuf, "%s%s/ab/ab_showall/%s/%s/%s/%d/%d/%d%s", beghtml, PATH_POSTMAN, stlang,
               (char *)user.cstr(), connid, avalidator, otropageid, cmdactual, endhtml);
      break;
      }       
    case CMD_CM:
      {
      sprintf (defbuf, "%s%s/cm//%s/%s/%s/%d/%d/%d%s", beghtml, PATH_POSTMAN, stlang,
               (char *)user.cstr(), connid, avalidator, otropageid, cmdactual, endhtml);
      break;
      }   
    case CMD_CM_FORWARD:
      {
      sprintf (defbuf, "%s%s/cm_forward//%s/%s/%s/%d/%d/%d/%ld%s", beghtml, PATH_POSTMAN, stlang,
               (char *)user.cstr(), connid, avalidator, otropageid, cmdactual, xatoldef (parm1, 1), endhtml);
      break;      
      }           
    case CMD_CM_REPLY:
      {
      sprintf (defbuf, "%s%s/cm_reply//%s/%s/%s/%d/%d/%d/%ld%s", beghtml, PATH_POSTMAN, stlang,
               (char *)user.cstr(), connid, avalidator, otropageid, cmdactual, xatoldef (parm1, 1), endhtml);
      break;      
      }       
    case CMD_MG_DISPLAY: case CMD_MG_DELDISPLAY:   
      {
      sprintf (defbuf, "%s%s/mg_display//%s/%s/%s/%d/%d/%d/%ld%s", beghtml, PATH_POSTMAN, stlang,
               (char *)user.cstr(), connid, avalidator, otropageid, cmdactual, xatoldef (parm1, 1), endhtml);      
      break;
      }
    case CMD_AB: case CMD_MB_INDEX: case CMD_OP: case CMD_MB_SHOW:  
    default:
      {
      break;
      }  
    }
         
  return (xstrdup(defbuf));                
  }  
  
                          
